@@ -8,9 +8,10 @@ from .Models import (
 )
 
 class HourObject:
-    def __init__(self, nh:list, ch:list) -> None:    
+    def __init__(self, nh:list, ch:list, dyn_ch:dict = {}) -> None:    
         self._nh = nh
         self._ch = ch
+        self._dyn_ch = dyn_ch
 
     @property
     def nh(self) -> list:
@@ -20,6 +21,10 @@ class HourObject:
     def ch(self) -> list:
         return self._ch
 
+    @property
+    def dyn_ch(self) -> dict:
+        return self._dyn_ch
+
 
 class Hoursselectionbase:
     def __init__(
@@ -31,17 +36,17 @@ class Hoursselectionbase:
         self._prices_tomorrow = None
         self._non_hours = []
         self._caution_hours = []
+        self._dynamic_caution_hours = {}
         self._absolute_top_price = self._set_absolute_top_price(absolute_top_price)
         self._cautionhour_type = cautionhour_type
         self._validate()
-        self.update()
     
     def _set_absolute_top_price(self, val) -> float:
         if val is None:
             return float("inf")
         if val <= 0:
             return float("inf")
-        return val
+        return float(val)
 
     def _validate(self):
         assert 0 < self._cautionhour_type <= 1
@@ -65,6 +70,14 @@ class Hoursselectionbase:
     def caution_hours(self, val):
         self._caution_hours = val
 
+    @property
+    def dynamic_caution_hours(self) -> dict:
+        return self._dynamic_caution_hours
+
+    @dynamic_caution_hours.setter
+    def dynamic_caution_hours(self, val):
+        self._dynamic_caution_hours = val
+
     @property
     def prices(self):
         return self._prices
@@ -94,25 +107,23 @@ class Hoursselectionbase:
 
         self.non_hours = []
         self.caution_hours = []
+        self.dynamic_caution_hours = {}
 
-        for h in hours_today.nh:
+        self.non_hours.extend(h for h in hours_today.nh if h >= hour)
+        self.caution_hours.extend(h for h in hours_today.ch if h >= hour)
+        self.non_hours.extend(h for h in hours_tomorrow.nh if h < hour)
+        self.caution_hours.extend(h for h in hours_tomorrow.ch if h < hour)
+
+        for h in hours_today.dyn_ch:
             if h >= hour:
-                self.non_hours.append(h)
+                self._dynamic_caution_hours[h] = hours_today.dyn_ch[h]
 
-        for h in hours_today.ch:
-            if h >= hour:
-                self.caution_hours.append(h)
-
-        for h in hours_tomorrow.nh:
+        for h in hours_tomorrow.dyn_ch:
             if h < hour:
-                self.non_hours.append(h)
-
-        for h in hours_tomorrow.ch:
-            if h < hour:
-                self.caution_hours.append(h)
+                self._dynamic_caution_hours[h] = hours_tomorrow.dyn_ch[h]
 
     def _update_internal(self, prices) -> HourObject:
-        ret = HourObject([], [])
+        ret = HourObject([], [], {})
         if prices is not None and len(prices) > 1:
 
             pricedict = self._create_dict(prices)
@@ -147,17 +158,22 @@ class Hoursselectionbase:
                 readyhours.nh.append(h)
                 if h in readyhours.ch:
                     readyhours.ch.remove(h)
+                if h in readyhours.dyn_ch.keys():
+                    readyhours.dyn_ch.pop(h)
         readyhours.nh.sort()
         return readyhours
 
-    def _rank_prices(self, hourdict: dict, normalized_hourdict: dict):
+    def _rank_prices(self, hourdict: dict, normalized_hourdict: dict) -> dict:
         ret = {}
         _maxval = max(hourdict.values())
         _max_normalized = max(normalized_hourdict.values())
         peaqstdev = _maxval/abs(_max_normalized/stat.stdev(normalized_hourdict.values()))
+        if peaqstdev < min(hourdict.values()):
+            peaqstdev = peaqstdev + min(hourdict.values())
         for key in hourdict:
             if hourdict[key] > peaqstdev:
-                ret[key] = {"val": hourdict[key], "permax": round(hourdict[key] / _maxval, 2)}
+                _permax = round(hourdict[key] / _maxval, 2)
+                ret[key] = {"val": hourdict[key], "permax": _permax}
         return ret
 
     def _create_dict(self, input: list):
@@ -172,15 +188,15 @@ class Hoursselectionbase:
 
     def _determine_hours(self, price_list: dict) -> HourObject:
         _nh = []
-        #_ch = {}
+        _dyn_ch = {}
         _ch = []
         for p in price_list:
             if float(price_list[p]["permax"]) <= self._cautionhour_type:
                 _ch.append(p)
-                #_ch[p] = round(abs(price_list[p]["permax"] - 1), 2)
+                _dyn_ch[p] = round(abs(price_list[p]["permax"] - 1), 2)
             else:
                 _nh.append(p)
-        return HourObject(_nh, _ch)
+        return HourObject(_nh, _ch, _dyn_ch)
     
     def _convert_none_list(self, lst:list) -> list:
         try:
@@ -216,3 +232,4 @@ class Hoursselectionbase:
             return ret
         except:
             return False
+
